class VirtualCardBillingAddress:
    def __init__(self, line1, line2, city, state, country, 
                postalCode) -> None:
        self.line1 = line1
        self.line2 = line2
        self.city = city
        self.state = state
        self.country = country
        self.postal_code = postalCode

class VirtualCard:
    def __init__(self, id, balance, cardNumber, 
                    cvv, cardName, cardType, expiry, 
                    status, valid, last4, blocked, frozen,
                    cardBrand, customerEmail, billingAddress) -> None:
        self.id = id
        self.balance = balance
        self.card_number = cardNumber
        self.card_name = cardName
        self.card_type = cardType
        self.card_brand = cardBrand
        self.expiry = expiry
        self.status = status
        self.valid = valid
        self.cvv = cvv
        self.last4 = last4
        self.blocked = blocked,
        self.frozen = frozen
        self.customer_email = customerEmail
        self.billing_address = VirtualCardBillingAddress(
            line1=billingAddress.get("line1"),
            line2=billingAddress.get("line2"),
            city=billingAddress.get("city"),
            state=billingAddress.get("state"),
            country=billingAddress.get("country"),
            postalCode=billingAddress.get("postalCode"),
        )

class VirtualCardTransaction:
    def __init__(self, id, amount, centAmount, type, currency, reference, cardId) -> None:
        self.id = id
        self.amount = amount
        self.cent_amount = centAmount
        self.type = type
        self.currency = currency
        self.reference = reference
        self.cardId = cardId

class VirtualCardUser:
    def __init__(self, id, customerEmail, firstName, lastName, 
                countryCode, phoneNumber, idType, idNumber, 
                city, state, country, zipCode, userPhoto, 
                customerId, line1, line2, kycPassed) -> None:
        self.id = id
        self.customer_email = customerEmail
        self.first_name = firstName
        self.last_name = lastName
        self.country_code = countryCode
        self.phone_number = phoneNumber
        self.id_type = idType
        self.id_number = idNumber
        self.city = city
        self.state = state
        self.country = country
        self.zip_code = zipCode
        self.user_photo = userPhoto
        self.customer_id = customerId
        self.line1 = line1
        self.line2 = line2
        self.kyc_passed = kycPassed



# from bitnob import CustomerApi
# from dotenv import load_dotenv
# load_dotenv() 

# customer_client = CustomerApi()

# def test_creating_customer():
#     body ={
#         "email": "precious@bitnob.com",
#         "firstName": "Precious",
#         "lastName": "Ndubueze",
#         "phone": "9064908234",
#         "countryCode": "+234"
#     }
#     response = customer_client.create_customer(body=body)
#     assert response["statusCode"] == 200


from bitnob.wallet.model import WalletAddress, Transaction
from bitnob.stablecoin.model import Receipt
from bitnob.customer.model import User
from bitnob.lightning.model import Invoice
from bitnob.hosted_checkout.model import Checkout, CheckoutStatus


class BitnobBadKeyError(Exception):
    """Exception raised for errors in the input salary.

    Attributes:
        salary -- input salary which caused the error
        message -- explanation of the error
    """

    def __init__(self):
        self.message = "API Key is not set"
        super().__init__(self.message)

class BitnobServerError(Exception):
    def __init__(self, *args):
        self.message = args[0]
        self.code = 500

    @property
    def response(self):
        return {"status": "error", "name": "BitnobServerError", "code": self.code, "message": self.message}


class BitnobUnauthorizedError(Exception):
    def __init__(self, *args):
        self.message = args[0]
        self.code = 401

    @property
    def response(self):
        return {"status": "error", "name": "BitnobUnauthorizedError", "code": self.code, "message": self.message}


class BitnobQueryError(Exception):
    def __init__(self, *args):
        self.message = args[0]
        self.code = 404

    @property
    def response(self):
        return {"status": "error", "name": "BitnobQueryError", "code": self.code, "message": self.message}


class BitnobRateLimitError(Exception):
    def __init__(self, *args):
        self.message = args[0]
        self.code = 429

    @property
    def response(self):
        return {"status": "error", "name": "P2PError", "code": self.code, "message": self.message}


class BitnobBadRequestError(Exception):
    def __init__(self, *args):
        self.message = args[0]
        self.code = 400

    @property
    def response(self):
        return {"status": "error", "name": "BitnobBadRequestError", "code": self.code, "message": self.message}

class BitnobRequiredParamError(Exception):
    def __init__(self, message):
        super().__init__(message)



from bitnob import USDT, USDC
usdc = USDC()
usdt = USDT()

def test_generate_usdc_address():
    body = {
        "chain": "TRX",
        "customerEmail": "precious@bitnob.com"
        }
    data = usdc.generate_address(body=body)
    assert data.address_type == "TRX"


# def test_generate_usdt_address():
#     body = {
#         "chain": "BSC",
#         "customerEmail": "precious@bitnob.com"
#         }
#     data = usdt.generate_address(body=body)
#     assert data.address_type == "BSC"


# def test_send_usdc():
#     body = {
#         "amount": 10,
#         "address": "4zMMC9srt5Ri5X14GAgXhaHii3GnPAEERYPJgZJDncDU",
#         "chain": "SOL",
#         "description": "string"
#     }
#     data = usdc.send(body=body)
#     assert data.amount == "10"
#     total = data.amount + data.fees
#     assert total == data.total_amount


# def test_send_usdt():
#     body = {
#         "amount": 10,
#         "address": "4zMMC9srt5Ri5X14GAgXhaHii3GnPAEERYPJgZJDncDU",
#         "chain": "BSC",
#         "description": "string"
#     }
#     data = usdt.send(body=body)
#     assert data.amount == "10"
#     total = data.amount + data.fees
#     assert total == data.total_amount


class User:
    def __init__(self, id, email, firstName, lastName, countryCode, phone) -> None:
        self.id = id
        self.email = email
        self.firstName = firstName
        self.lastName = lastName
        self.countryCode = countryCode
        self.phone = phone
class Checkout:
    def __generate_user_object(self, data):
        return User(id=data["id"], 
                        email=data["email"], 
                        firstName=data["firstName"], 
                        lastName=data["lastName"], 
                        countryCode=data["countryCode"], 
                        phone=data["phone"]
                )

    def __init__(self, id, reference, amount,  
                sat_amount, sat_amount_paid, expires_at,
                lightning_expires_at, status, callback_url, success_url, checkout_type,
                description, address, lightning, companyName, companyLogo, customer) -> None:
        self.id = id
        self.reference = reference
        self.amount = amount
        self.sat_amount_paid = sat_amount_paid
        self.lightning_expires_at = lightning_expires_at
        self.expires_at = expires_at
        self.sat_amount = sat_amount
        self.callback_url = callback_url
        self.checkout_type = checkout_type
        self.status = status
        self.success_url = success_url
        self.description = description
        self.address = address
        self.lightning = lightning
        self.customer = self.__generate_user_object(data=customer)
        self.companyName = companyName
        self.companyLogo = companyLogo

class CheckoutStatus:
    def __init__(self, id, sat_amount, sat_amount_paid, status) -> None:
        self.id = id
        self.sat_amount_paid = sat_amount_paid
        self.sat_amount = sat_amount
        self.status = status




from bitnob.base import Bitnob, pagination_filter
from .model import Invoice
from bitnob.wallet.model import Transaction


class Lightning(Bitnob):

    def __generate_invoice_object(self, data):
        return Invoice(
            description = data.get("description"),
            tokens=data.get("tokens"),
            request=data.get("request")
        )
    
    def __generate_transaction_object(self, data):
        return Transaction(            
            reference = data.get("reference"),
            description = data.get("description"),
            amount = data.get("amount"),
            sat_amount = data.get("satAmount"),
            fees = data.get("fees"),
            btc_fees = data.get("btcFees"),
            sat_fees = data.get("satFees"),
            action = data.get("action"),
            transaction_type = data.get("type"),
            status = data.get("status"),
            id=data.get("id"),
            invoice_id=data.get("invoiceId"),
            channel=data.get("channel"),
            address=data.get("address"),
            payment_request=data.get("paymentRequest"),
            spot_price=data.get("spotPrice"),
            hash=data.get("hash"),
            confirmations=data.get("confirmations")
        )
    
    def create_invoice(self, body:dict):
        """
        Creating lightning invoice for customer

        body = {
            "description": "xxxxxxxxx",
            "tokens": 1000,
            "private": false,
            "is_including_private_channels": false,
            "is_fallback_included": true,
            "customerEmail": "customer@gmail.com"
        }

        required_data = "description", "tokens", "customerEmail"

        - POST Request
        """
        required_data = ["description", "tokens", "customerEmail"]
        self.check_required_datas(required_data, body)

        response = self.send_request("POST", "wallets/ln/createinvoice", json=body)
        return self.__generate_invoice_object(data=response["data"])


    def pay_invoice(self, body:dict):
        """
        Paying lightning invoice

        body = {
            "request": "lntb10u1ps3z4vqpp5lhvr5hdhtw2z32f88dfym5hw2pk9a0a2gr5g0x85ccdy3hpwuneqdpsd4hkueteypehgmmswvsxummwwdjkuum9yphkugrnw4hxgctecqzpgxqr23sfppqtthagr80s0jruguvg38g78pgl8nwp32esp5wvx8sus7a4g66ujtw2zzvyj49zd2sfw5msh3jhaes2vm3z6ga84s9qyyssqm73qauemzqw3tz0equ7cfs2l8xfcgqzuusgg6rplp4hprv83cde97fhh5vxglsqfzwzpms24d7xrqc77cjv32tny82nktfhkz699awcqwdhwur",
            "reference": "reference"
        }

        required_data = "request", "reference"

        - POST Request
        """
        required_data = ["request", "reference"]
        self.check_required_datas(required_data, body)

        response = self.send_request("POST", "wallets/ln/pay", json=body)
        return self.__generate_transaction_object(data=response["data"])
    
    def initiate_payment(self, invoice_request):
        """
        Initiate payment on lighting network.

        invoice_request = "lntb10u1psny337pp5km35drx473py8ucup33k0qhrql0ll7cg528c9999eyssp8l0l7xsdpsd4hkueteypehgmmswvsxummwwdjkuum9yphkugrnw4hxgctecqzpgxqr23sfppq00dkh7zckw5d7xgzkk0tctcamlzu6fnrsp53d3kuz79s7akxj9hx0ucnd5fzpd5dlsmwxu7dcja0ksqguqkkm7q9qyyssqyr0lgw260u7pqyjfuwp4azszhugt9m353c9vfq36qltxnqe40qm3dxz4vrqxjyp0utyhvr3p3rrvjpuc0jfts3yd0y02ckqhnx9zp8gpzrazdt"

        - POST Request
        """
        
        body = {"request" : invoice_request}
        return self.send_request("POST", "wallets/ln/initiatepayment", json=body)
    
    def decode_payment_request(self, invoice_request):
        """
        Decode Payment Request

        invoice_request = "lntb10u1psny337pp5km35drx473py8ucup33k0qhrql0ll7cg528c9999eyssp8l0l7xsdpsd4hkueteypehgmmswvsxummwwdjkuum9yphkugrnw4hxgctecqzpgxqr23sfppq00dkh7zckw5d7xgzkk0tctcamlzu6fnrsp53d3kuz79s7akxj9hx0ucnd5fzpd5dlsmwxu7dcja0ksqguqkkm7q9qyyssqyr0lgw260u7pqyjfuwp4azszhugt9m353c9vfq36qltxnqe40qm3dxz4vrqxjyp0utyhvr3p3rrvjpuc0jfts3yd0y02ckqhnx9zp8gpzrazdt"

        - POST Request
        """
        body = {"request" : invoice_request}
        return self.send_request("POST", "wallets/ln/decodepaymentrequest", json=body)
    
    def get_invoice(self, invoice_id):
        """
        Getting lightning invoice
        invoice_id = "1b7605e0cdfe8b0267fe377f5ecb7d068375b551e4a626880e668e1aec23bf64"

        - POST Request
        """
        body = {"id" : invoice_id}
        return self.send_request("POST", "wallets/ln/getinvoice", json=body)
    
    def list_invoices(self, **kwargs):
        """
        Getting addresses attached to company

        - POST Request
        """
        url_params = None
        if kwargs != {}:
            url_params = pagination_filter(**kwargs)
        return self.send_request("GET", f"/wallets/ln/getinvoices/?{url_params}",)


# {
#   "status": true,
#   "message": "lightning invoice successfully created",
#   "data": {
#     "description": "jsjsjs",
#     "request": "lntb10u1p3p9t5ypp53lxdgsqtem24ytrded2aq5x90yvm8fn3nl9h6dxr0u6lqrfeys3qdq2dfek5um2wvcqzpgxqr23sfppqppth2s03ya2ks4979rwg39ddc0env8tesp5sl60vw2k82l32xune9u6dlhge9gqtstk7ujzfqge3vdpk59w6gfs9qyyssqaalvv6z2qx88e7hakh9t33pryeyr7jcsvn8crsfv0j7y5gvzts3qflf7htn744ywdzuneqmkp0zyv8mjcrfnfkslha0cjxejleltrhgqjzlpvy",
#     "tokens": "1000"
#   }
# }
class Invoice:
    def __init__(self, description, request, tokens) -> None:
        self.description = description
        self.request = request
        self.tokens = tokens

# "data": {
#         "reference": "skjrjfr",
#         "amount": "0.93",
#         "fees": "0",
#         "btcFees": "0",
#         "btcAmount": "0.0000193",
#         "satFees": "0",
#         "satAmount": "1930",
#         "spotPrice": "47979.76",
#         "action": "send_bitcoin",
#         "type": "debit",
#         "status": "pending",
#         "channel": "lightning",
#         "paymentRequest": "lntb19300n1ps5xxgepp5r6w7hpqzfe5f5eg9ep98u0243qtnqn9e7xmwvjynzf25kyyghr9sdpsd4hkueteypehgmmswvsxummwwdjkuum9yphkugrnw4hxgctecqzpgxqr23sfppq97tmvgjq9r4s24047zukwvqrmz5g720wsp5hjxvud80p7yc8p58vdgj8d30c7t8nv5d9rnd59vftccxqvvwk3ns9qyyssq87pa3vxsntqturd5r6n9g57gv964r80q0wcnjcakzsuuqk3xkpgpxul304v7n82xhju8xcjkgckywwhfqzshj5w9q7apat08nm2nh8cpczpu6n",
#         "description": null,
#         "address": null,
#         "hash": null,
#         "confirmations": null,
#         "invoiceId": null,
#         "id": "5c3683d3-5e73-4d15-a910-74c8cba6c8ef",
#         "createdAt": "2021-09-16T10:15:26.711Z",
#         "updatedAt": "2021-09-16T10:15:26.711Z"
#     }



class Receipt:
    def __init__(self, id, reference, description, amount, fees, action, receipt_type, status) -> None:
        self.reference =reference
        self.description = description
        self.amount = amount
        self.fees = fees
        self.action = action
        self.receipt_type = receipt_type
        self.status = status
        self.id=id
    
    @property
    def cent_amount(self):
        self.amount * 100
    
    @property
    def cent_fees(self):
        self.fees * 100
    
    @property
    def total_amount(self):
        """
        Total amount debited from user
        """
        return self.amount + self.fees
    
    @property
    def total_cents(self):
        """
        Total cents debited from user
        """
        return self.centAmount + self.centFees


import os
import hmac
import secrets
import requests

from hashlib import sha512
from requests.exceptions import HTTPError, ConnectionError

from .exceptions import exception_class, BitnobBadKeyError, BitnobRequiredParamError


class Bitnob():
    """
    Base Bitnob class
    """

    def __init__(self):
        self.BITNOB_LIVE_URL = 'https://api.bitnob.co/api/v1/'
        self.BITNOB_SANDBOX_URL = 'https://sandboxapi.bitnob.co/api/v1/'
        self.api_key = os.environ.get("BITNOB_API_KEY")
        self.production = os.environ.get("BITNOB_PRODUCTION")
        self.base_url = os.environ.get("BITNOB_BASE_URL")
        if self.api_key is None:
            raise BitnobBadKeyError()
        backupUrl = (
           self.BITNOB_SANDBOX_URL if self.production is False else self.BITNOB_LIVE_URL
        )
        getattr(self, "base_url", backupUrl)

    def send_request(self, method, path, **kwargs):
        """
        Create a request stub
        :param method:
        :param path:
        :param kwargs:
        :return:
        """
        options = {
            "GET": requests.get,
            "POST": requests.post,
            "PUT": requests.put,
            "PATCH": requests.patch,
            "DELETE": requests.delete,
        }
        url = "{}{}".format(self.base_url, path)
        headers = {
            "content-type": "application/json",
            "Authorization": "Bearer {}".format(self.api_key),
        }
        try:
            response = options[method](url, headers=headers, **kwargs)
            data = response.json()
            try:
                data["statusCode"]
                exception = exception_class.get(data["statusCode"])
                raise exception(data["message"])
            except KeyError:
                return data
        except (HTTPError, ConnectionError) as e:
            return e
    
    def check_required_datas(self, required_data, passed_param):
        """
        function to check required params 
        """
        for key in required_data:
            if key not in passed_param.keys():
                added_message = "The following are required: " + ",".join(required_data)
                message = f'{key} is required! ' + added_message
                raise BitnobRequiredParamError(message)
                

def pagination_filter(**kwargs):
    return "&".join([f"{k}={v}" for k, v in kwargs.items()])

def webhook_authenication(request):
    """Validate signed requests."""
    api_signature = request.headers.get("x-bitnob-signature")
    secret = os.environ.get("BITNOB_WEBHOOK_SECRET")
    computed_sig = hmac.new(
        key=secret.encode("utf-8"), msg=request.body, digestmod=sha512
    ).hexdigest()
    return computed_sig == api_signature
        


from bitnob.base import Bitnob, pagination_filter
from .model import Checkout, CheckoutStatus


class HostedCheckout(Bitnob): 

    def __generate_checkout_object(self, data):
        return Checkout(
            id=data["id"],
            reference = data["reference"],
            amount = data["amount"],
            sat_amount_paid = data["sat_amount_paid"],
            lightning_expires_at = data["lightning_expires_at"],
            expires_at = data["expires_at"],
            sat_amount = data["sat_amount"],
            callback_url = data["callback_url"],
            checkout_type = data["type"],
            status = data["status"],
            success_url = data["success_url"],
            description = data["description"],
            address = data["address"],
            lightning = data["lightning"],
            customer=data["customer"],
            companyName=data["companyName"],
            companyLogo=data["companyName"],
        )
    
    def __generate_checkout_status_object(self, data):
        return CheckoutStatus(
                        sat_amount_paid=data["sat_amount_paid"],
                        sat_amount = data["sat_amount"],
                        status = data["status"]
                )
    
    def create_checkout(self, body:dict):
        """
        Create Checkout

        body = {
            amount: 10,
            description: "description"
            customerEmail: "customer@gmail.com",
            notificationEmail: "CustomerfirstName",
            callbackUrl: "htpps://domain.com/callbackUrl",
            successUrl: "htpps://domain.com/successUrl",
            reference: "reference"
        }

        - POST Request
        """
        required_data =  ["amount", "description", "customerEmail", "notificationEmail", "callbackUrl", "successUrl", "reference"]
        self.check_required_datas(required_data, body)
        
        response = self.send_request("POST", "checkout", json=body)
        return self.__generate_checkout_object(data=response.get("data"))

    def list_checkouts(self, **kwargs):
        """
        Listing checkouts

        order = (optional) Result order. Accepted values: `DESC` (default), ASC
        page = (optional) Result page.
        take = (optional) Number of results per request. Accepted values: 0 - 100. Default 10
        
        - GET Request
        """
        url_params = None
        if kwargs != {}:
            url_params = pagination_filter(**kwargs)
        response = self.send_request("GET", f"checkout/?{url_params}")
        data = response.get("data")
        return [self.__generate_checkout_object(user_data) for user_data in data]
    
    def get_checkout_status(self, checkout_id):
        """
        Get Checkout Status

        - GET Request
        """
        response =  self.send_request("GET", f"checkout/status/{checkout_id}")
        return self.__generate_checkout_status_object(data=response.get("data"))


    def get_checkout_info(self, checkout_id):
        """
        Get Checkout Status

        - GET Request
        """
        response =  self.send_request("GET", f"checkout/info/{checkout_id}")
        return self.__generate_checkout_object(data=response.get("data"))
    


class User:
    def __init__(self, id, email, firstName, lastName, countryCode, phone) -> None:
        self.id = id
        self.email = email
        self.firstName = firstName
        self.lastName = lastName
        self.countryCode = countryCode
        self.phone = phone


class LNURL:
    def __init__(self,         
        id, 
        tld, 
        identifier, 
        identifier_type, 
        ln_url, 
        ln_url_QR,
        sat_min_sendable,
        sat_max_sendable) -> None:
        self.id = id
        self.tld = tld
        self.identifier = identifier
        self.identifier_type = identifier_type
        self.ln_url = ln_url
        self.ln_url_QR = ln_url_QR
        self.sat_min_sendable = sat_min_sendable
        self.sat_max_sendable = sat_max_sendable


from .exceptions import *

exception_class = {
    400: BitnobBadRequestError,
    500: BitnobServerError,
    401: BitnobUnauthorizedError,
    429: BitnobRateLimitError,
    404: BitnobQueryError,
}



from bitnob.base import Bitnob, pagination_filter
from .model import User

class Customer(Bitnob): 

    def __generate_user_object(self, data):
        return User(id=data["id"], 
                        email=data["email"], 
                        firstName=data["firstName"], 
                        lastName=data["lastName"], 
                        countryCode=data["countryCode"], 
                        phone=data["phone"]
                )
    
    def create_customer(self, body:dict):
        """
        Creating your customer on the Bitnob platform
        body = {
            "email": "customer@gmail.com",
            "firstName": "CustomerfirstName",
            "lastName": "CustomerlastName",
            "phone": "9xxxxxxxx",
            "countryCode": "+234"
        }

        - POST Request
        """
        required_data = ["email", "firstName", "lastName", "phone", "countryCode"]
        self.check_required_datas(required_data, body)
        
        response = self.send_request("POST", "customers", json=body)
        return self.__generate_user_object(data=response.get("data"))



    def list_customers(self, **kwargs):
        """
        Listing all customer

        order = (optional) Result order. Accepted values: `DESC` (default), ASC
        page = (optional) Result page.
        take = (optional) Number of results per request. Accepted values: 0 - 100. Default 10
        
        - GET Request
        """
        url_params = None
        if kwargs != {}:
            url_params = pagination_filter(kwargs=kwargs)
        response = self.send_request("GET", f"customers/?{url_params}")
        data = response["data"]["customers"]
        return [self.__generate_user_object(user_data) for user_data in data]

    
    def get_customer(self, customer_id):
        """
        Getting User using customer_id

        - GET Request
        """
        response =  self.send_request("GET", f"customers/{customer_id}")
        return self.__generate_user_object(data=response.get("data"))
    
    def get_customer_by_email(self, email):
        """
        Getting a customer using their email
        email = customer@email.com

        - POST Request
        """
        body = {"email": email}
        response =  self.send_request("POST", "customers/fetch_customer", json=body)
        return self.__generate_user_object(data=response.get("data"))
    
    def update_customer(self, body:dict, customer_id):
        """
        Updating User using customer_id
        body = {
            "email": "customer@gmail.com",
            "firstName": "CustomerfirstName",
            "lastName": "CustomerlastName",
            "phone": "9xxxxxxxx",
            "countryCode": "+234"
        }

        - PUT Request
        """
        response =  self.send_request("PUT", f"customers/{customer_id}", json=body)
        return self.__generate_user_object(data=response.get("data"))


from bitnob.base import Bitnob, pagination_filter
from bitnob.model import WalletAddress, Transaction


class Onchain(Bitnob): 

    def __generate_address_object(self, data):
        return WalletAddress(
            address = data.get("address"),
            address_type=data.get("addressType"),
            label=data.get("label")
        )
    
    def __generate_transaction_object(self, data):
        return Transaction(            
            reference = data.get("reference"),
            description = data.get("description"),
            amount = data.get("amount"),
            sat_amount = data.get("satAmount"),
            fees = data.get("fees"),
            btc_fees = data.get("btcFees"),
            sat_fees = data.get("satFees"),
            action = data.get("action"),
            transaction_type = data.get("type"),
            status = data.get("status"),
            id=data.get("id"),
            invoice_id=data.get("invoiceId"),
            channel=data.get("channel"),
            address=data.get("address"),
            payment_request=data.get("paymentRequest"),
            spot_price=data.get("spotPrice"),
            hash=data.get("hash"),
            confirmations=data.get("confirmations")
        )

    
    def send_bitcoin(self, body:dict):
        """
        Sending bitcoin

        body = {
            "satoshis": 3000,
            "address": "tb1q9h0yjdupyfpxfjg24rpx755xrplvzd9hz2nj7v",
            "customerEmail": "customer@email.com",
            "description": "lorem ipsum",
            "priorityLevel": "high"
        }

        required_data = "satoshis", "address", "customerEmail"

        - POST Request
        """
        required_data = ["satoshis", "address", "customerEmail"]
        self.check_required_datas(required_data, body)

        response = self.send_request("POST", "wallets/send_bitcoin", json=body)
        return self.__generate_transaction_object(data=response["data"])

    def generate_address(self, body:dict):
        """
        Generate Address for Customer

        body = {
            "label": "purchase xbox",
            "customerEmail": "customer@gmail.com"
        }

        - POST Request
        """
        required_data = ["customerEmail"]
        self.check_required_datas(required_data, body)

        response = self.send_request("POST", "addresses/generate", json=body)
        return self.__generate_address_object(data=response["data"])
    
    def list_addresses(self, **kwargs):
        """
        Getting addresses attached to company

        - POST Request
        """
        url_params = None
        if kwargs != {}:
            url_params = pagination_filter(**kwargs)
        response =  self.send_request("GET", f"/addresses/?{url_params}",)
        data = response.get("data")
        return [self.__generate_address_object(address_data) for address_data in data]
    
    def get_recommeded_btc_fes(self):
        return self.send_request("GET", "recommended-fees/btc")


class WalletAddress:
    def __init__(self,address, address_type, label=None) -> None:
        self.address = address
        self.address_type = address_type
        self.label = label

class Transaction:
    def __init__(self, id, reference, amount, 
                fees, btc_fees, sat_fees, 
                sat_amount, spot_price, action,
                transaction_type, status, channel, payment_request, 
                description, address, hash,
                confirmations, invoiceId) -> None:
        self.id = id
        self.reference = reference
        self.amount = amount
        self.btc_fees = btc_fees
        self.fees = fees
        self.sat_fees = sat_fees
        self.sat_amount = sat_amount
        self.spot_price = spot_price
        self.transaction_type = transaction_type
        self.status = status
        self.channel = channel
        self.action = action
        self.payment_request = payment_request
        self.description = description
        self.address = address
        self.hash = hash
        self.confirmations = confirmations
        self.invoice_id = invoiceId






from bitnob import Onchain

onchain = Onchain()

def test_generate_btc_address():
    body = {
        "customerEmail": "precious@bitnob.com"
        }
    data = onchain.generate_address(body=body)
    assert data.address_type == "bech32"


from bitnob.base import Bitnob, pagination_filter
from .model import LNURL


class LNURL(Bitnob): 
    def __generate_ln_url_object(self, data):
        return LNURL(
            id=data["id"], 
            tld = data["tld"],
            identifier = data["identifier"],
            identifier_type = data["identifier_type"],
            ln_url = data["ln_url"],
            ln_url_QR = data["ln_url_QR"],
            sat_min_sendable = data["sat_min_sendable"],
            sat_max_sendable = data["sat_max_sendable"]
        )

    def create_lnurl(self, body):
        """
        Create LNURL/Address

        body = {
            "identifier": "satoshi",
            "tld": "bitcoin.org",
            "identifierType": "username",
            "customerEmail": "satoshi@bitcoin.org",
            "satMinSendable": 1,
            "satMaxSendable": 1000000
        }

        required_data = "identifier", "tld", "customerEmail"

        - POST Request
        """
        required_data = ["identifier", "tld", "customerEmail"]
        self.check_required_datas(required_data, body)
        response = self.send_request("POST", "lnurl", json=body)
        return self.__generate_ln_url_object(data=response.get("data"))

    def decode_lnurl(self, encoded_lnurl):
        """
        Decode LnUrl

        encoded_lnurl

        - POST Request
        """
        body = {
            "encodedLnurl": encoded_lnurl
        }
        response = self.send_request("POST", "lnurl", json=body)
        return response

    def pay_lnurl(self, body:dict):
        """
        Paying lnurl

        body = {
            "encodedLnUrl": "LNURL1DP68GURN8GHJ7UM5V9NKJMN894SHQ6FWVEKX7AM9WF6X7UPW0PUH5TEWWAJKCMPDDDHX7AMW9ACXKTNP8QMNSE3WV4JRXC3SX3JRGERZXYERJCFNVG6KZTMVDE6HYMRS9AEKZAR0WD5XJJE6N7Y",
            "satoshis": 1000,
            "reference": "reference",
            "customerEmail": "satoshi@bitcoin.org",
            "comment": "comment",
        }

        required_data = "encodedLnUrl", "reference", "satoshis", "customerEmail"

        - POST Request
        """
        required_data = ["encodedLnUrl", "reference", "satoshis", "customerEmail"]
        self.check_required_datas(required_data, body)

        response = self.send_request("POST", "lnurl/paylnurl", json=body)
        return response
    

    def create_ln_withdrawal(self, body:dict):
        """
        create ln-withdrawal

        body = {
            "customerEmail": "satoshi@bitcoin.org",
            "description": "Have some gifts for the holidays",
            "satoshis": 1000
        }

        required_data = "description", "satoshis", "customerEmail"

        - POST Request
        """
        required_data =  ["description", "satoshis", "customerEmail"]
        self.check_required_datas(required_data, body)

        response = self.send_request("POST", "lnurl/createLnUrlWithdrawal", json=body)
        return response
    
    def decode_lnaddress(self, ln_address):
        """
        Decode lnaddress

        encoded_lnurl

        - POST Request
        """
        body = {
            "lnAddress": ln_address
        }
        response = self.send_request("POST", "lnurl/decodelnaddress", json=body)
        return response
    
    def pay_lnaddress(self, body:dict):
        """
        Paying lnaddress

        body = {
            "lnAddress": "satoshi@bitcoin.org",
            "satoshis": 10,
            "reference": "testpayment",
            "customerEmail": "satoshi@bitnob.com"
        }

        required_data = "encodedLnUrl", "reference", "satoshis", "customerEmail"

        - POST Request
        """
        required_data = ["lnAddress", "reference", "satoshis", "customerEmail"]
        self.check_required_datas(required_data, body)

        response = self.send_request("POST", "lnurl/paylnaddress", json=body)
        return response
    
    def list_lnurls(self, **kwargs):
        """
        Listing lnurls

        order = (optional) Result order. Accepted values: `DESC` (default), ASC
        page = (optional) Result page.
        take = (optional) Number of results per request. Accepted values: 0 - 100. Default 10
        
        - GET Request
        """
        url_params = None
        if kwargs != {}:
            url_params = pagination_filter(kwargs=kwargs)
        response = self.send_request("GET", f"lnurl/?{url_params}")
        data = response["data"]["lnUrls"]
        return [self.__generate_ln_url_object(ln_data) for ln_data in data]
    
    def get_lnurl(self, lnurl_id):
        """
        Getting LNURL using lnurl_id

        - GET Request
        """
        response =  self.send_request("GET", f"lnurl/{lnurl_id}")
        return response
    
    def get_lnurl_by_identifier(self, username):
        """
        Getting a user lnurl by their username
        username = "satoshi"

        - GET Request
        """
        response =  self.send_request("GET", f"lnurl/fetchlnurl/{username}")
        return self.__generate_ln_url_object(data=response.get("data"))
    
    def update_lnurl(self, body:dict, lnurl_id):
        """
        Updating LNURL
        body = {
            "identifier": "satoshi",
            "tld": "bitcoin.org",
            "identifierType": "username",
            "customerEmail": "satoshi@bitcoin.org",
            "satMinSendable": 1,
            "satMaxSendable": 1000000
        }

        - PUT Request
        """
        response =  self.send_request("PUT", f"lnurl/{lnurl_id}", json=body)
        return self.__generate_ln_url_object(data=response.get("data"))


from bitnob.lightning.services import Lightning
from bitnob.customer.services import Customer
from bitnob.onchain.services import Onchain
from bitnob.wallet.services import Wallet
from bitnob.virtual_card.services import VirtualCardClient
from bitnob.stablecoin.services import USDC, USDT
from bitnob.hosted_checkout.services import HostedCheckout
from bitnob.ln_url.services import LNURL
from bitnob.base import webhook_authenication
from bitnob import model


from bitnob.base import Bitnob, pagination_filter


class Wallet(Bitnob): 
    def wallet_detail(self):
        """
        Retrive company wallet details

        - GET Request
        """
        return self.send_request("GET", "wallets")

    def get_transaction(self, transaction_id):
        """
        Getting a transaction based on transaction_id 

        - GET Request
        """
        return self.send_request("GET", f"/transactions/{transaction_id}")
    
    def list_transactions(self, **kwargs):
        """
        Listing all transactions

        order = (optional) Result order. Accepted values: `DESC` (default), ASC
        page = (optional) Result page.
        take = (optional) Number of results per request. Accepted values: 0 - 100. Default 10
        
        - GET Request
        """
        url_params = None
        if kwargs != {}:
            url_params = pagination_filter(**kwargs)
        return self.send_request("GET", f"transactions/?{url_params}")
    
    def swap_btc_to_usd(self, amount:int):
        """
        Swap BTC to USD
        amount = 0.00001

        - POST Request
        """
        body = {
            "amount" : amount
        }

        return self.send_request("POST", "wallets/swap-bitcoin-usd", json=body)
    
    def swap_usd_to_btc(self, amount:int):
        """
        Swap USD to BTC
        amount = 30

        - POST Request
        """
        body = {
            "amount" : amount
        }

        return self.send_request("POST", "wallets/swap-usd-bitcoin", json=body)


from bitnob.base import Bitnob
from abc import ABC, abstractmethod
from bitnob.stablecoin.model import Receipt
from bitnob.wallet.model import WalletAddress

class StableCoin(ABC):

    @abstractmethod
    def send(self):
        pass

    @abstractmethod
    def generate_address(self):
        pass

    def generate_address_object(self, data):
        return WalletAddress(
            address = data.get("address"),
            address_type=data.get("addressType"),
            label=data.get("label")
        )
    
    def generate_receipt_object(self, data):
        return Receipt(
            reference = data.get("reference"),
            description = data.get("description"),
            amount = data.get("amount"),
            fees = data.get("fees"),
            action = data.get("action"),
            receipt_type = data.get("type"),
            status = data.get("status"),
            id=data.get("id")
        )
    



class USDC(StableCoin, Bitnob): 
    
    def send(self, body:dict):
        """
        Sending USDC

        body = {
            amount: 3000,
            address: "tb1q9h0yjdupyfpxfjg24rpx755xrplvzd9hz2nj7v",
            chain: "BSC"
            description: "lorem ipsum",
        } 

        - POST Request
        """
        required_data = ["amount", "address", "chain"]
        self.check_required_datas(required_data, body)

        response = self.send_request("POST", "wallets/send-usdc", json=body)
        return self.generate_receipt_object(data=response["data"])


    def generate_address(self, body:dict):
        """
        Generate Address for Customer

        body = {
            "label": "purchase xbox",
            "customerEmail": "customer@gmail.com",
            "chain": "BSC"
        }

        - POST Request
        """
        required_data = ["customerEmail", "chain"]
        self.check_required_datas(required_data, body)

        response =  self.send_request("POST", "addresses/generate/usdc", json=body)
        return self.generate_address_object(data=response["data"])

class USDT(StableCoin, Bitnob): 
    
    def send(self, body:dict):
        """
        Sending USDC

        body = {
            amount: 3000,
            address: "tb1q9h0yjdupyfpxfjg24rpx755xrplvzd9hz2nj7v",
            chain: "BSC",
            description: "lorem ipsum",
        } 

        - POST Request
        """
        required_data = ["amount", "address", "chain"]
        self.check_required_datas(required_data, body)

        response = self.send_request("POST", "wallets/send-usdt", json=body)
        return self.generate_receipt_object(data=response["data"])

    def generate_address(self, body:dict):
        """
        Generate Address for Customer

        body = {
            "label": "purchase xbox",
            "customerEmail": "customer@gmail.com"
            "chain": "BSC"
        }

        - POST Request
        """
        required_data = ["customerEmail", "chain"]
        self.check_required_datas(required_data, body)

        response =  self.send_request("POST", "addresses/generate/usdt", json=body)
        return self.generate_address_object(data=response["data"])



import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

setup(
    name="bitnob",
    version="1.2.6",
    description="Python SDK for the Bitnob\"s API",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/bitnob/bitnob_python_sdk",
    author="Bitnob",
    author_email="info@bitnob.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    install_requires=["requests"],
    packages=find_packages(),
    python_requires=">=3.6"
)


from bitnob.base import Bitnob, pagination_filter
from bitnob.virtual_card.model import VirtualCard, VirtualCardTransaction, VirtualCardUser


class VirtualCardClient(Bitnob):

    def __generate_virtual_card_object(self, data):
        return VirtualCard(
            id = data.get("id"),
            cardNumber=data.get("cardNumber"),
            cardBrand=data.get("cardBrand"),
            cvv= data.get("cvv2"),
            balance=data.get("balance"),
            cardName = data.get("cardName"),
            cardType = data.get("cardType"),
            expiry = data.get("expiry"),
            status = data.get("status"),
            valid = data.get("valid"), 
            last4=data.get("last4"),
            blocked=data.get("blocked"),
            frozen=data.get("frozen"),
            customerEmail=data.get("customerEmail"),
            billingAddress=data.get("billingAddress")
        )
    
    def __generate_card_trans_object(self, data):
        return VirtualCardTransaction(
            id = data.get("id"),
            cardId = data.get("cardId"),
            amount = data.get("amount"),
            centAmount = data.get("centAmount"),
            type = data.get("type"),
            currency = data.get("currency"),
            reference = data.get("reference"),

        )
    
    def __generate_card_user_object(self, data):
        return VirtualCardUser(
            id = data.get("id"),
            customerEmail = data.get("customerEmail"),
            firstName = data.get("firstName"),
            lastName = data.get("lastName"),
            countryCode = data.get("countryCode"),
            phoneNumber = data.get("phoneNumber"),
            idType = data.get("idType"),
            idNumber = data.get("idNumber"),
            city = data.get("city"),
            state = data.get("state"),
            country = data.get("country"),
            zipCode = data.get("zipCode"),
            userPhoto = data.get("userPhoto"),
            customerId = data.get("customerId"),
            line1 = data.get("line1"),
            line2 = data.get("line2") or None, 
            kycPassed = data.get("kycPassed")
        )

    def register_card_user(self, body:dict):
        """
        Registering a Card User to access Virtual Card

        body = {
            customerEmail: "customer@gmail.com",
            idNumber: "478haomaaa",
            idType: "NIN",
            firstName: "John",
            lastName: "Doe",
            city: "New York",
            state: "Alabama"
            country: "Nigeria",
            zipCode: "100001", 
            userPhoto:"image.jpg"
            phoneNumber: "08023456789"
            line1: "38 West Street",
            line2: "Apt. 1",
        }

        - POST Request
        """
        required_data = ["customerEmail", "idNumber", "idNumber", "firstName", "lastName", "city", "state", "country", "zipCode", "line1", "phoneNumber", "userPhoto"]
        self.check_required_datas(required_data, body)
        
        response = self.send_request("POST", "virtualcards/registercarduser", json=body)
        return self.__generate_card_user_object(data=response["data"])
        
    def create_card(self, body:dict):
        """
        Create Card for customer

        email = customer@gmail.com
        """
        response = self.send_request("POST", "virtualcards/create", json=body)
        return self.__generate_virtual_card_object(data=response["data"])
    
    def top_up(self, body:dict):
        """
        Top Up Card
        body = {
            customerEmail: "customer@gmail.com",
            cardId: "d1cbc518-95d6-4bf2-a0f4-e6a33ee09603",
            amount: 1000
        }
        """
        response = self.send_request("POST", "virtualcards/topup", json=body)
        return self.__generate_card_trans_object(data=response["data"])        

    def withdraw(self, body:dict):
        """
        Withdraw from Card for customer

        body = {
            customerEmail: "customer@gmail.com",
            cardId: "d1cbc518-95d6-4bf2-a0f4-e6a33ee09603",
            amount: 1000
        }
        """
        response = self.send_request("POST", "virtualcards/withdraw", json=body)
        return self.__generate_card_trans_object(data=response["data"])        
    
    def mock_transaction(self, body:dict):
        """
        Mock transaction(works only on sandbox)
        body = {
            cardId: "d1cbc518-95d6-4bf2-a0f4-e6a33ee09603",
            amount: 1000,
            type: deduct
        }
        """
        return self.send_request("POST", "virtualcards/mock-transaction", json=body)

    def freeze_card(self, card_id:str):
        """
        Freeze Card

        card_id = d1cbc518-95d6-4bf2-a0f4-e6a33ee09603
        """
        body = {
            "cardId" : card_id
        }
        response = self.send_request("POST", "virtualcards/freeze", json=body)
        return self.__generate_virtual_card_object(data=response["data"])
    
    def unfreeze_card(self, card_id:str):
        """
        Unfreeze Card

        card_id = d1cbc518-95d6-4bf2-a0f4-e6a33ee09603
        """
        body = {
            "cardId" : card_id
        }
        response = self.send_request("POST", "virtualcards/unfreeze", json=body)
        return self.__generate_virtual_card_object(data=response["data"])
    
    def block_card(self, card_id:str):
        """
        Block Card

        card_id = d1cbc518-95d6-4bf2-a0f4-e6a33ee09603
        """
        body = {
            "cardId" : card_id
        }
        response = self.send_request("POST", "virtualcards/block", json=body)
        return self.__generate_virtual_card_object(data=response["data"])
    
    def unblock_card(self, card_id:str):
        """
        Unblock Card

        card_id = d1cbc518-95d6-4bf2-a0f4-e6a33ee09603
        """
        body = {
            "cardId" : card_id
        }
        response = self.send_request("POST", "virtualcards/unblock", json=body)
        return self.__generate_virtual_card_object(data=response["data"])
    
    def terminate_card(self, card_id:str):
        """
        Unfreeze Card

        card_id = d1cbc518-95d6-4bf2-a0f4-e6a33ee09603
        """
        body = {
            "cardId" : card_id
        }
        return self.send_request("POST", "virtualcards/terminate", json=body)

    def list_cards(self, **kwargs):
        """
        Listing all cards

        order = (optional) Result order. Accepted values: `DESC` (default), ASC
        page = (optional) Result page.
        take = (optional) Number of results per request. Accepted values: 0 - 100. Default 10
        
        - GET Request
        """
        url_params = None
        if kwargs != {}:
            url_params = pagination_filter(**kwargs)
        response = self.send_request("GET", f"virtualcards/cards/?{url_params}")
        data = response["data"]["cards"]
        return [self.__generate_virtual_card_object(card_data) for card_data in data]
    
    def list_card_transactions(self, card_id, **kwargs):
        """
        Listing card transactions

        card_id = d1cbc518-95d6-4bf2-a0f4-e6a33ee09603

        order = (optional) Result order. Accepted values: `DESC` (default), ASC
        page = (optional) Result page.
        take = (optional) Number of results per request. Accepted values: 0 - 100. Default 10
        
        - GET Request
        """
        url_params = None
        if kwargs != {}:
            url_params = pagination_filter(**kwargs)
        response = self.send_request("GET", f"virtualcards/cards/{card_id}/transactions/?{url_params}")
        data = response["data"]["cardTransactions"]
        return [self.__generate_card_trans_object(card_transactions) for card_transactions in data]
    
    def list_transactions(self, **kwargs):
        """
        Listing all transactions

        order = (optional) Result order. Accepted values: `DESC` (default), ASC
        page = (optional) Result page.
        take = (optional) Number of results per request. Accepted values: 0 - 100. Default 10
        
        - GET Request
        """
        url_params = None
        if kwargs != {}:
            url_params = pagination_filter(**kwargs)
        response = self.send_request("GET", f"virtualcards/cards/transactions/?{url_params}")
        data = response["data"]["cardTransactions"]
        return [self.__generate_card_trans_object(card_transactions) for card_transactions in data]

    def get_card(self, card_id):
        """
        Getting Card using card_id
        card_id = d1cbc518-95d6-4bf2-a0f4-e6a33ee09603

        - GET Request
        """
        response = self.send_request("GET", f"virtualcards/cards/{card_id}")
        return self.__generate_virtual_card_object(data=response["data"])
    
    def list_card_users(self, **kwargs):
        """
        Listing all Card Users

        order = (optional) Result order. Accepted values: `DESC` (default), ASC
        page = (optional) Result page.
        take = (optional) Number of results per request. Accepted values: 0 - 100. Default 10
        
        - GET Request
        """
        url_params = None
        if kwargs != {}:
            url_params = pagination_filter(**kwargs)
        response = self.send_request("GET", f"/virtualcards/users?{url_params}")
        data = response["data"]["cardUsers"]
        return [self.__generate_card_user_object(card_users) for card_users in data]